/font defs: ---/,/end font defs ---/c\
%-------------------- PostScript (long names) font defs: -----------------\
\\font\\bbf=Times-Bold at 10pt\
\\font\\vbbf=Times-Bold at 12pt\
\\font\\smrm=Times-Roman at 6pt\
\\font\\brm=Times-Roman at 10pt\
\\font\\rm=Times-Roman at 8pt\
\\font\\it=Times-Italic at 8pt\
\\font\\tt=Courier at 8pt\
% Used only for \copyright, replacing plain TeX macro.\
\\font\\sym=Symbol at 7pt\
\\def\\copyright{{\\sym\\char'323}}\
%-------------------- end font defs ---------------------------------
